from flask import Flask, render_template, redirect, request, Blueprint
import os

from model import Team, Game, Question, Round, Answer
from peewee import fn

from views import verify_request_data, build_current_round_json, send_reply, SOCKET
from utils import ERROR_LOG

game = Blueprint("game", "game")

def form_verify_game(p_request):
    return verify_request_data(Game, "name", p_request.form.get("gamename"))

def form_verify_question(p_request):
    return verify_request_data(Question, "uuid", p_request.form.get("question_id"))

@game.route("/game", methods=["POST"])
def render_game_view(game=None):
    if game is None:
        game = form_verify_game(request)
        if game is None:
            return redirect("/management")

    build_current_round_json(game)
    return send_reply(render_template("game.jinja", game=game, untaken_question_list=Question.select().where(Question.taken==False), error_list=ERROR_LOG.fetch()))

@game.route("/remove_round", methods=["POST"])
def remove_round():
    game = form_verify_game(request)
    if game is None:
        return redirect("/management")

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)

    if not question.used:
        round = Round.get_or_none(question=question, game=game)
        if round is None:
            ERROR_LOG.add("Internal error. Could not find round (game='%s', question='%s'). That should not happen :( Perhaps someone tempered with the database" % (gamename, question_id))
            return render_game_view(game)

        round.delete_instance()
        question.taken = False
        question.save()
    else:
        ERROR_LOG.add("A question that has been used aleady cannot be deleted from a game")
        return render_game_view(game)

    return render_game_view(game)

@game.route("/create_round", methods=["POST"])
def create_round():
    game = form_verify_game(request)
    if game is None:
        return redirect("/management")

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)

    if not question.taken:
        round = Round.create(game=game, question=question)
        round.save()

        question.taken = True
        question.save()

        game.save()
    else:
        ERROR_LOG.add("The question is already taken: '%s' (%s)" % (question.question, question.uui))

    return render_game_view(game)

@game.route("/create_random_round", methods=["POST"])
def create_random_round():
    game = form_verify_game(request)
    if game is None:
        return redirect("/management")

    random_question = Question.select().where(Question.taken==False).order_by(fn.Random())
    if len(random_question) <= 0:
        ERROR_LOG.add("Es gibt keine verfügbaren Fragen mehr :(")
        return render_game_view(game)
    question = random_question[0]
    question.taken = True
    question.save()

    r = Round.create(question=question, game=game)
    r.save()
    game.save()

    return render_game_view(game)

@game.route("/show_answer", methods=["POST"])
def show_answer():
    game = form_verify_game(request)
    if game is None:
        return redirect("/management")

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)
            
    question.used = True

    answer_id = request.form.get("answer_id")
    if answer_id is None:
        ERROR_LOG.add("Request is incomplete. Missing id of answer")
        return render_game_view(game)

    r = Round.get_or_none(question=question, game=game)
    if r is None:
        ERROR_LOG.add("Internal error. Could not find round (game='%s', question='%s'). That should not happen :( Perhaps someone tempered with the database" % (gamename, question_id))
        return render_game_view(game)

    r.show_answer(answer_id)
    question.save()

    os.system("mpv frontend/audio/frage.mp3 &")

    return render_game_view(game)

@game.route("/hide_answer", methods=["POST"])
def hide_answer():
    game = form_verify_game(request)
    if game is None:
        return  redirect("/management")

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)

    question.used = True
    
    answer_id = request.form.get("answer_id")
    if answer_id is None:
        ERROR_LOG.add("Request is incomplete. Missing id of answer")
        return render_game_view(game)

    r = Round.get_or_none(question=question, game=game)
    if r is None:
        ERROR_LOG.add("Internal error. Could not find round (game='%s', question='%s'). That should not happen :( Perhaps someone tempered with the database" % (gamename, question_id))
        return render_game_view(game)

    r.hide_answer(answer_id)
    question.save()

    return render_game_view(game)

@game.route("/uncredit_round", methods=["POST"])
def uncredit_round():
    game = form_verify_game(request)
    if game is None:
        return  redirect("/management")

    team = request.form.get("team")
    if team not in ["left", "right"]:
        ERROR_LOG.add("Unknown Team-Identifier. Either use 'left' or 'right. Given '%s'" % team)
        return render_game_view(game)

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)

    r = Round.get_or_none(question=question, game=game)
    if r is None:
        ERROR_LOG.add("Internal error. Could not find round (game='%s', question='%s'). That should not happen :( Perhaps someone tempered with the database" % (gamename, question_id))
        return render_game_view(game)

    if team == "left":
        game.points_left -= r.points
    else:
        game.points_right -= r.points

    game.save()

    return render_game_view(game)

@game.route("/credit_round", methods=["POST"])
def credit_round():
    game = form_verify_game(request)
    if game is None:
        return  redirect("/management")

    team = request.form.get("team")
    if team not in ["left", "right"]:
        ERROR_LOG.add("Unknown Team-Identifier. Either use 'left' or 'right. Given '%s'" % team)
        return render_game_view(game)

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)

    r = Round.get_or_none(question=question, game=game)
    if r is None:
        ERROR_LOG.add("Internal error. Could not find round (game='%s', question='%s'). That should not happen :( Perhaps someone tempered with the database" % (gamename, question_id))
        return render_game_view(game)

    if team == "left":
        game.points_left += r.points
    else:
        game.points_right += r.points

    game.save()

    return render_game_view(game)
     
@game.route("/resolve_round", methods=["POST"])
def resolve_round():
    game = form_verify_game(request)
    if game is None:
        return  redirect("/management")

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)

    question.used = True

    for answer in question.answers:
        answer.show()
        answer.save()
    
    question.save()

    os.system("mpv frontend/audio/frage.mp3 &")

    return render_game_view(game)

@game.route("/next_round", methods=["POST"])
def next_round():
    game = form_verify_game(request)
    if game is None:
        return redirect("/management")

    game.next_round()
    game.save()

    return render_game_view(game)

@game.route("/jump_to_round", methods=["POST"])
def jump_to_round():
    game = form_verify_game(request)
    if game is None:
        return redirect("/management")

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)

    round = Round.get_or_none(game=game, question=question)
    if round is None:
        ERROR_LOG.add("Internal error. Could not find round (game='%s', question='%s'). That should not happen :( Perhaps someone tempered with the database" % (gamename, question_id))
        return render_game_view(game)

    game.jump_to_round(round)

    return render_game_view(game)

@game.route("/error_team", methods=["POST"])
def error_team():
    game = form_verify_game(request)
    if game is None:
        return  redirect("/management")

    team = request.form.get("team")
    if team not in ["left", "right"]:
        ERROR_LOG.add("Unknown Team-Identifier. Either use 'left' or 'right. Given '%s'" % team)
        return render_game_view(game)

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)

    r = Round.get_or_none(question=question, game=game)
    if r is None:
        ERROR_LOG.add("Internal error. Could not find round (game='%s', question='%s'). That should not happen :( Perhaps someone tempered with the database" % (gamename, question_id))
        return render_game_view(game)

    if team == "left":
        r.error_left += 1
    else:
        r.error_right += 1

    r.save()

    os.system("mpv frontend/audio/failure.mp3 &")

    return render_game_view(game)


@game.route("/unerror_team", methods=["POST"])
def unerror_team():
    game = form_verify_game(request)
    if game is None:
        return  redirect("/management")

    team = request.form.get("team")
    if team not in ["left", "right"]:
        ERROR_LOG.add("Unknown Team-Identifier. Either use 'left' or 'right. Given '%s'" % team)
        return render_game_view(game)

    question = form_verify_question(request)
    if question is None:
        return render_game_view(game)

    r = Round.get_or_none(question=question, game=game)
    if r is None:
        ERROR_LOG.add("Internal error. Could not find round (game='%s', question='%s'). That should not happen :( Perhaps someone tempered with the database" % (gamename, question_id))
        return render_game_view(game)

    if team == "left":
        r.error_left -= 1
        if r.error_left < 0:
            r.error_left = 0
    else:
        r.error_right -= 1
        if r.error_right < 0:
            r.error_right = 0

    r.save()

    return render_game_view(game)

    